import { extname, resolve, dirname, normalize, join } from 'path';
// Valid file extensions
const imageExts = ['.png', '.jpg', '.jpeg', '.gif', '.tiff'];
const jsonExts = ['.json', '.json5'];
const fallbackExts = [...imageExts, ...jsonExts];

class FallbackResourcePlugin {
  // Applying plugin to compiler
  apply(compiler : any) {
    compiler.resolverFactory.hooks.resolver.for("normal").tap("fallbackResource", (resolver : any) => {

      // Modify normal resolver function resolve
      resolver.hooks.resolve.tapAsync('fallbackResource', async (params : any, innerContext : any, cb : Function) => {

        // Check if resolving file is image
        const ext = extname(params.request);
        if (!fallbackExts.includes(ext)) {
          cb(null, null);
          return;
        }


        // Check if image is valid
        const resolvedPath = resolve(
          normalize(dirname(params.context.issuer)),
          normalize(params.request)
        );
        resolver.fileSystem.stat(resolvedPath, (err : Error) => {

          if (!err) {
            // Skip if valid
            cb(null, null);
            return;
          }

          const isImageResource = imageExts.includes(ext);
          const defaultResource = isImageResource ? 'cleanImage.png' : 'defaultJson.json'

          if (isImageResource) {
            // Push warning if not valid and resolve as clean image
            console.warn(`\n${params.context.issuer}\r\nFile '${params.request}' not found.\r\n Please add this file!`)
          }

          const result = {
            path: join(__dirname, defaultResource)
          };
          cb(null, result);
        });
      });
    });
  }
}

export default FallbackResourcePlugin;
